'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function containsStringsOnly (array) {
  return array
    .map((current) => typeof current === 'string')
    .reduce((previous, current) => previous && current)
}

function validateDisabled (disabled) {
  if (disabled === undefined) {
    return true
  }
  if (typeof disabled !== 'boolean') {
    console.warn('[Vue-clickoutside-handler:] provided disabled option', disabled, 'is not a boolean.');
    return false
  }
  return true
}

function validateExcluded (excluded) {
  if (!excluded) {
    return true
  }
  if (!Array.isArray(excluded)) {
    console.warn('[Vue-clickoutside-handler:] provided excluded', excluded, 'is not an array.');
    return false
  }
  if (!containsStringsOnly(excluded)) {
    console.warn('[Vue-clickoutside-handler:] provided excluded', excluded, 'contains non string values.');
    return false
  }
  return true
}

function validateHandler (handler) {
  if (typeof handler !== 'function') {
    console.warn('[Vue-clickoutside-handler:] provided handler', handler, 'is not a function.');
    return false
  }
  return true
}

const SUPPORTED_OPTIONS = ['handler', 'excluded', 'disabled'];

function validate (options) {
  options = options || {};

  if (typeof options === 'function') {
    return true
  }

  for (const opt of Object.keys(options)) {
    if (!SUPPORTED_OPTIONS.includes(opt)) {
      console.warn('[Vue-clickoutside-handler:] provided option', opt, 'is not a supported.');
      return false
    }
  }

  return validateHandler(options.handler) &&
    validateExcluded(options.excluded) &&
    validateDisabled(options.disabled)
}

function executorFactory (el, opts) {
  let handler = opts;
  let excluded, disabled;
  if (typeof opts !== 'function') {
    handler = opts.handler;
    excluded = opts.excluded;
    disabled = opts.disabled;
  }

  return (event) => {
    event.stopPropagation();

    const excludedElements = Array.from(document.querySelectorAll(excluded));

    if (!el.contains(event.target) &&
      !isClickOnExcluded(event.target, excludedElements) &&
      !disabled
    ) {
      handler();
    }
  }
}

function isClickOnExcluded (target, excludedElements) {
  return excludedElements
    .map((excludedEl) => excludedEl.contains(target))
    .reduce((previousValue, currentValue) => previousValue || currentValue, false)
}

function bind (el, binding) {
  if (!validate(binding.value)) {
    return
  }

  const handleOutsideClick = executorFactory(el, binding.value);

  el.__vueOutsideHandler__ = { handleOutsideClick };
  // Register click/touchstart event listeners on the whole page
  document.addEventListener('click', handleOutsideClick);
  document.addEventListener('touchstart', handleOutsideClick);
}

function unbind (el) {
  document.removeEventListener('click', el.__vueOutsideHandler__.handleOutsideClick);
  document.removeEventListener('touchstart', el.__vueOutsideHandler__.handleOutsideClick);
  delete el.__vueOutsideHandler__;
}

function update (el, binding, vnode, oldVnode) {
  unbind(el);
  bind(el, binding);
}

const VueClickOut = {
  bind,
  update,
  unbind
};

exports.VueClickOut = VueClickOut;
